@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2002-2005 
+ * Copyright (C) 2002-2009
  * Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -32,6 +32,7 @@
 #include "ucclam.h"
 #include "uccroco.h"
 #include "uctidy.h"
+#include "cookies.h"
 
 #include "web_site.h"
 
@@ -142,6 +143,9 @@ static gboolean uc_check_content_type_bad_extensions (const UCLinkType type);
 static UCLinkProperties *uc_check_link_already_checked (const gchar * url);
 static gboolean uc_check_get_w3c_html_validate (UCLinkProperties * prop);
 static gboolean uc_check_get_w3c_css_validate (UCLinkProperties * prop);
+static void uc_check_display_list_current_item (const UCLinkProperties *
+						prop);
+
 
 gboolean
 uc_check_html_is_valid (const gchar * buffer)
@@ -258,7 +262,9 @@ uc_check_status_is_good (const gchar * status)
   return (status[0] == '2' || (!strcmp (status, UC_STATUS_CODE_FTP_OK) ||
 			       !strcmp (status, UC_STATUS_CODE_REDIRECTED) ||
 			       !strcmp (status,
-					UC_STATUS_CODE_MOVED_PERMANENTLY)
+					UC_STATUS_CODE_REDIRECTED_SEE_OTHER)
+			       || !strcmp (status,
+					   UC_STATUS_CODE_MOVED_PERMANENTLY)
 			       || !strcmp (status,
 					   UC_STATUS_CODE_GOOD_EMAIL_LINK)));
 }
@@ -283,6 +289,7 @@ uc_check_status_is_bad (const gchar * status)
 	  && status[0] != '9'
 	  && status[0] != '8'
 	  && strcmp (status, UC_STATUS_CODE_REDIRECTED)
+	  && strcmp (status, UC_STATUS_CODE_REDIRECTED_SEE_OTHER)
 	  && strcmp (status, UC_STATUS_CODE_MOVED_PERMANENTLY)
 	  && strcmp (status, "503")
 	  && strcmp (status, UC_STATUS_CODE_TIMEOUT)
@@ -549,6 +556,66 @@ uc_check_get_link_type_icon_path (const UCLinkType link_type,
   return UC_PIXMAPS_DIR "/list_unknown.png";
 }
 
+static void
+uc_check_display_list_current_item (const UCLinkProperties * prop)
+{
+  gchar *lm_date = NULL;
+  gchar *label = NULL;
+  gchar *value = NULL;
+  GtkTreePath *treepath = NULL;
+  GtkTreeIter c_iter;
+  UCURLsUserActions *action = NULL;
+
+
+  /*
+   * we do not display meta links nor nodes to remove from
+   * bookmarks */
+  if (prop->link_type == LINK_TYPE_META || prop->to_delete)
+    return;
+
+  lm_date = g_hash_table_lookup (prop->header, UC_LAST_MODIFIED);
+  if (lm_date == NULL)
+    lm_date = "-";
+
+  action = uc_application_get_urls_user_action (prop->user_action);
+
+  label = uc_utils_string_format4display (prop->label,
+					  UC_LABEL_DISPLAY_MAX_LEN);
+  value = uc_utils_string_format4display (prop->link_value,
+					  UC_LABEL_DISPLAY_MAX_LEN);
+
+  gtk_tree_store_append (treestore, &c_iter, NULL);
+
+  gtk_tree_store_set (treestore, &c_iter,
+		      UID_COLUMN, prop->uid,
+		      LINK_STATUS_ICON_COLUMN,
+		      prop->status_icon,
+		      LINK_ICON_COLUMN,
+		      prop->link_icon,
+		      SECURITY_ALERT_ICON_COLUMN,
+		      (uc_check_is_security_alert (prop)) ?
+		      security_alert_icon : empty_icon,
+		      W3C_ALERT_ICON_COLUMN,
+		      (uc_check_is_w3c_alert (prop)) ?
+		      w3c_alert_icon : empty_icon,
+		      ACTION_COLUMN, action->label,
+		      LABEL_COLUMN,
+		      label, URL_COLUMN, value, LM_COLUMN, lm_date, -1);
+
+  gtk_tree_view_get_cursor (treeview, &treepath, NULL);
+
+  if (!treepath)
+    treepath = gtk_tree_path_new_from_string ("0");
+  else
+    gtk_tree_path_next (treepath);
+
+  gtk_tree_view_set_cursor (GTK_TREE_VIEW (treeview), treepath, NULL, FALSE);
+
+  gtk_tree_path_free (treepath), treepath = NULL;
+  g_free (value), value = NULL;
+  g_free (label), label = NULL;
+}
+
 /**
  * uc_check_display_list:
  * @list: #GList of the links to display.
@@ -564,6 +631,7 @@ void
 uc_check_display_list (GList * list, gchar * path)
 {
   UCLinkProperties *prop = NULL;
+  UCURLsUserActions *action = NULL;
   GList *item = NULL;
   gchar *lm_date = NULL;
   gchar *label = NULL;
@@ -590,6 +658,8 @@ uc_check_display_list (GList * list, gchar * path)
       prop->treeview_path = (prop->depth_level == 0) ?
 	g_strdup_printf ("%u", pos) : g_strdup_printf ("%s:%u", path, pos);
 
+      action = uc_application_get_urls_user_action (prop->user_action);
+
       lm_date = g_hash_table_lookup (prop->header, UC_LAST_MODIFIED);
       if (lm_date == NULL)
 	lm_date = "-";
@@ -615,6 +685,7 @@ uc_check_display_list (GList * list, gchar * path)
 			  W3C_ALERT_ICON_COLUMN,
 			  (uc_check_is_w3c_alert (prop)) ?
 			  w3c_alert_icon : empty_icon,
+			  ACTION_COLUMN, action->label,
 			  LABEL_COLUMN,
 			  label, URL_COLUMN, value, LM_COLUMN, lm_date, -1);
 
@@ -648,6 +719,7 @@ uc_check_display_list_with_status (GList * list, GtkTreeIter iter,
 {
   GList *item = NULL;
   UCLinkProperties *prop = NULL;
+  UCURLsUserActions *action = NULL;
   gchar *status_code = NULL;
   gchar *lm_date = NULL;
   gchar *label = NULL;
@@ -707,6 +779,9 @@ uc_check_display_list_with_status (GList * list, GtkTreeIter iter,
 	      if (lm_date == NULL)
 		lm_date = "-";
 
+	      action =
+		uc_application_get_urls_user_action (prop->user_action);
+
 	      label =
 		uc_utils_string_format4display (prop->label,
 						UC_LABEL_DISPLAY_MAX_LEN);
@@ -727,6 +802,7 @@ uc_check_display_list_with_status (GList * list, GtkTreeIter iter,
 				  W3C_ALERT_ICON_COLUMN,
 				  (uc_check_is_w3c_alert (prop)) ?
 				  w3c_alert_icon : empty_icon,
+				  ACTION_COLUMN, action->label,
 				  LABEL_COLUMN,
 				  label, URL_COLUMN, value,
 				  LM_COLUMN, lm_date, -1);
@@ -759,10 +835,12 @@ uc_check_display_list_with_type (GList * list, GtkTreeIter iter,
 {
   GList *item = NULL;
   UCLinkProperties *prop = NULL;
+  UCURLsUserActions *action = NULL;
   gchar *lm_date = NULL;
   gchar *label = NULL;
   gchar *value = NULL;
 
+
   item = g_list_first (list);
   while (item != NULL)
     {
@@ -782,6 +860,8 @@ uc_check_display_list_with_type (GList * list, GtkTreeIter iter,
 	  if (lm_date == NULL)
 	    lm_date = "-";
 
+	  action = uc_application_get_urls_user_action (prop->user_action);
+
 	  label =
 	    uc_utils_string_format4display (prop->label,
 					    UC_LABEL_DISPLAY_MAX_LEN);
@@ -802,6 +882,7 @@ uc_check_display_list_with_type (GList * list, GtkTreeIter iter,
 			      W3C_ALERT_ICON_COLUMN,
 			      (uc_check_is_w3c_alert (prop)) ?
 			      w3c_alert_icon : empty_icon,
+			      ACTION_COLUMN, action->label,
 			      LABEL_COLUMN,
 			      label, URL_COLUMN, value, LM_COLUMN, lm_date,
 			      -1);
@@ -833,10 +914,12 @@ uc_check_display_list_with_protocol (GList * list, GtkTreeIter iter,
 {
   GList *item = NULL;
   UCLinkProperties *prop = NULL;
+  UCURLsUserActions *action = NULL;
   gchar *lm_date = NULL;
   gchar *label = NULL;
   gchar *value = NULL;
 
+
   item = g_list_first (list);
   while (item != NULL)
     {
@@ -857,6 +940,8 @@ uc_check_display_list_with_protocol (GList * list, GtkTreeIter iter,
 	  if (lm_date == NULL)
 	    lm_date = "-";
 
+	  action = uc_application_get_urls_user_action (prop->user_action);
+
 	  label =
 	    uc_utils_string_format4display (prop->label,
 					    UC_LABEL_DISPLAY_MAX_LEN);
@@ -877,6 +962,7 @@ uc_check_display_list_with_protocol (GList * list, GtkTreeIter iter,
 			      W3C_ALERT_ICON_COLUMN,
 			      (uc_check_is_w3c_alert (prop)) ?
 			      w3c_alert_icon : empty_icon,
+			      ACTION_COLUMN, action->label,
 			      LABEL_COLUMN,
 			      label, URL_COLUMN, value, LM_COLUMN, lm_date,
 			      -1);
@@ -946,7 +1032,6 @@ uc_check_url_get_content (UCLinkProperties * prop,
   gchar *cookies = NULL;
   GArray *response = NULL;
   gchar *path = NULL;
-  const gchar *begin = NULL;
   gchar *ret = NULL;
   gboolean is_local = FALSE;
 
@@ -958,7 +1043,7 @@ uc_check_url_get_content (UCLinkProperties * prop,
   is_local = !strcmp (prop->h_name, uc_project_get_current_host ());
   referer = uc_check_build_referer_field ();
   cookies = (is_local && uc_project_get_cookies_accept ())?
-    uc_project_cookies_get_header_field (prop->path) : g_strdup ("");
+    uc_cookies_get_header_field (prop->path) : g_strdup ("");
 
   /* build request */
   request = (uc_project_get_use_proxy ())?
@@ -992,7 +1077,7 @@ uc_check_url_get_content (UCLinkProperties * prop,
   /* get HTTP response */
   response =
     uc_server_get_response (prop->protocol, hostname, atoi (prop->port),
-			    request);
+			    request, FALSE);
   g_free (request), request = NULL;
 
   /* exit if no result */
@@ -1000,30 +1085,26 @@ uc_check_url_get_content (UCLinkProperties * prop,
     return NULL;
 
   /* save content in cache */
-  /* FIXME */
   if (response->data != NULL)
     {
       if (prop->is_downloadable)
 	{
 	  FILE *fd;
+	  const gchar *begin = NULL;
 	  gchar *m = NULL;
 
 	  path = uc_utils_convert_uid2file (prop->uid);
 	  fd = fopen (path, "wb");
 	  g_free (path), path = NULL;
 
-	  m = g_malloc0 (response->len + 1);
+	  begin = m = g_malloc0 (response->len + 1);
 	  memcpy (m, response->data, response->len);
 
-	  if ((begin = strstr (m, "\r\n\r\n")))
-	    {
-	      begin += 4;
+	  begin += prop->header_size + 4;
+	  fwrite (begin, 1, response->len - prop->header_size - 4, fd);
+	  fclose (fd);
 
-	      fwrite (begin, 1, strlen (begin), fd);
-	      fclose (fd);
-
-	      ret = g_strdup (begin);
-	    }
+	  ret = g_strdup (begin);
 
 	  g_free (m), m = NULL;
 	}
@@ -1087,11 +1168,13 @@ uc_check_suspend_continue (void)
     {
       WSENS ("bt_ignore_progress_dialog", FALSE);
       gtk_button_set_label (GTK_BUTTON (widget), _("Continue"));
+      uc_application_progress_dialog_set_modal (FALSE);
     }
   else
     {
       WSENS ("bt_ignore_progress_dialog", TRUE);
       gtk_button_set_label (GTK_BUTTON (widget), _("Suspend"));
+      uc_application_progress_dialog_set_modal (TRUE);
     }
 
   while (uc_check_suspend_get_value () && !uc_check_cancel_get_value ())
@@ -1125,7 +1208,6 @@ uc_check_url_get_header (UCLinkProperties * prop)
   gchar str[UC_HEADER_STATUS_CODE_LEN + 1] = { 0 };
   guint i = 0;
   gboolean is_local = FALSE;
-  const gchar *cmd = "HEAD";
 
   is_local = !strcmp (prop->h_name, uc_project_get_current_host ());
 
@@ -1147,11 +1229,9 @@ uc_check_url_get_header (UCLinkProperties * prop)
 
       UC_UPDATE_UI;
 
-    get_header:
-
       referer = uc_check_build_referer_field ();
       cookies = (is_local && uc_project_get_cookies_accept ())?
-	uc_project_cookies_get_header_field (prop->path) : g_strdup ("");
+	uc_cookies_get_header_field (prop->path) : g_strdup ("");
       uri = (uc_project_get_use_proxy ())?
 	g_strdup_printf ("%s://%s%s%s", prop->protocol, prop->h_name,
 			 prop->path, prop->args) : g_strdup_printf ("%s%s",
@@ -1160,14 +1240,13 @@ uc_check_url_get_header (UCLinkProperties * prop)
 								    prop->
 								    args);
       /* build request */
-      request = g_strdup_printf ("%s %s HTTP/1.0\r\n"
+      request = g_strdup_printf ("GET %s HTTP/1.0\r\n"
 				 "User-Agent: %s\r\n"
 				 "Host: %s\r\n"
 				 "%s"
 				 "%s"
 				 "%s"
 				 "\r\n",
-				 cmd,
 				 uri,
 				 UC_USER_AGENT, prop->h_name,
 				 (uc_project_get_auth_line ())?
@@ -1182,7 +1261,7 @@ uc_check_url_get_header (UCLinkProperties * prop)
   /* get response */
   array =
     uc_server_get_response (prop->protocol, prop->h_name, atoi (prop->port),
-			    request);
+			    request, TRUE);
   g_free (request), request = NULL;
 
   /* exit if no result */
@@ -1193,24 +1272,9 @@ uc_check_url_get_header (UCLinkProperties * prop)
       return NULL;
     }
 
-  if (!strcmp (cmd, "GET"))
-    {
-      gint ptr = 0;
-
-      if (!(ptr = strcspn (array->data, "<")))
-	{
-	  g_array_free (array, TRUE), array = NULL;
-	  return NULL;
-	}
-      g_free (response), response = NULL;
-      response = g_new0 (gchar, ptr + 1);
-      memcpy (response, array->data, ptr);
-    }
-  else
-    {
-      response = g_new0 (gchar, array->len + 1);
-      memcpy (response, array->data, array->len);
-    }
+  response = g_new0 (gchar, array->len + 1);
+  memcpy (response, array->data, array->len);
+  response[array->len] = '\0';
 
   /* create hash table for header fields */
   header = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
@@ -1227,14 +1291,6 @@ uc_check_url_get_header (UCLinkProperties * prop)
       /* get HTTP header code */
       memcpy (&str, &response[UC_HEADER_STATUS_CODE_FIRST_POS],
 	      UC_HEADER_STATUS_CODE_LEN);
-      /* If the server doesn(t honor HEAD request, try
-       * GET */
-      if (!strcmp (str, "400") || !atoi (str))
-	{
-	  g_hash_table_destroy (header), header = NULL;
-	  cmd = "GET";
-	  goto get_header;
-	}
       g_hash_table_replace (header, g_strdup (UC_HEADER_STATUS),
 			    g_strdup (str));
 
@@ -1254,16 +1310,20 @@ uc_check_url_get_header (UCLinkProperties * prop)
 	      g_strstrip (key_value[0]);
 	      g_strstrip (key_value[1]);
 
-	      g_hash_table_replace (header,
-				    cookie_field =
-				    g_utf8_strdown (key_value[0], -1),
-				    g_strdup (key_value[1]));
+	      /* Status header field has already been set */
+	      if (strcasecmp (key_value[0], UC_HEADER_STATUS) != 0)
+		{
+		  g_hash_table_replace (header,
+					cookie_field =
+					g_utf8_strdown (key_value[0], -1),
+					g_strdup (key_value[1]));
 
-	      /* If this is cookie field, register the cookie */
-	      if (is_local && !uc_check_cancel_get_value () &&
-		  uc_project_get_cookies_accept () &&
-		  !strcmp (cookie_field, UC_SET_COOKIE))
-		uc_project_cookies_add (prop, key_value[1]);
+		  /* If this is cookie field, register the cookie */
+		  if (is_local && !uc_check_cancel_get_value () &&
+		      uc_project_get_cookies_accept () &&
+		      !strcmp (cookie_field, UC_SET_COOKIE))
+		    uc_cookies_add (prop, key_value[1]);
+		}
 	    }
 
 	  g_strfreev (key_value), key_value = NULL;
@@ -1360,7 +1420,9 @@ uc_check_register_link (const gchar * normalized_url, UCLinkProperties * lp)
 
   /* append link to the already checked temporary 
    * list */
-  uc_lists_already_checked_links_append (normalized_url, (gpointer) lp);
+  if (uc_lists_already_checked_links_append (normalized_url, (gpointer) lp))
+    if (uc_project_get_check_is_main ())
+      uc_check_display_list_current_item (lp);
 
   /* we do not register bookmark's folders */
   if (lp->link_type != LINK_TYPE_BOOKMARK_FOLDER &&
@@ -1833,7 +1895,7 @@ uc_check_get_virii (UCLinkProperties * prop)
 
   if ((clam = uc_ucclam_new ()))
     {
-      ret = uc_ucclam_scan (clam, path);
+      ret = uc_ucclam_scan (clam, path, &prop->virname);
       uc_ucclam_free (clam), clam = NULL;
     }
 
@@ -2105,6 +2167,7 @@ uc_check_link_properties_node_new (void)
   prop->link_type = LINK_TYPE_NONE;
   prop->link_icon = NULL;
   prop->status_icon = NULL;
+  prop->user_action = NULL;
   prop->link_value = NULL;
   prop->url = NULL;
   prop->normalized_url = NULL;
@@ -2124,6 +2187,7 @@ uc_check_link_properties_node_new (void)
   prop->similar_links_parents = NULL;
   prop->bad_extensions = NULL;
   prop->virii = FALSE;
+  prop->virname = NULL;
   prop->w3c_valid = TRUE;
   prop->tags = NULL;
   prop->is_parsable = FALSE;
@@ -2600,8 +2664,8 @@ uc_check_link_get_properties (const guint depth, gchar * current_host,
   /* if we chroot the check, initialize the directory to
    * chroot in */
   if (depth == 0)
-    uc_project_set_chroot_path ((uc_project_get_check_chroot ())? prop->
-				path : NULL);
+    uc_project_set_chroot_path ((uc_project_get_check_chroot ())? prop->path :
+				NULL);
 
   /* if we are in chroot mode (just this directory and its subdirectories,
    * or just local links) */
