@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2005 
+ * Copyright (C) 2002-2007
  * Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -71,6 +71,7 @@ enum _UCTcpStatus
  * @thread_id: current thread.
  * @thread_end: %TRUE when the main thread is not yet running.
  * @thread_ftp_end: %TRUE when the FTP thread is not yet running.
+ * @header_request: %TRUE when only header is needed.
  * @timeout: %TRUE if a timeout occured while retreiving/sending data.
  * @tcp_state: A #UCTcpState value to describe the connection state.
  * @tcp_status: A #UCTcpState value to describe the connection status.
@@ -95,6 +96,7 @@ struct _UCConn
   gboolean thread_end;
   gboolean thread_ftp_end;
   gboolean timeout;
+  gboolean header_request;
   UCTcpState tcp_state;
   UCTcpStatus tcp_status;
   gchar *hostname;
@@ -134,7 +136,8 @@ static gnutls_certificate_client_credentials xcred_gnutls;
 static void *connect_thread (void *data);
 static void *read_thread (void *data);
 static UCConn *uc_conn_new (const gchar * proto, gchar * host,
-			    guint port, gchar * buffer);
+			    guint port, gchar * buffer,
+			    gboolean header_request);
 static void uc_conn_disconnect (UCConn * conn);
 static void uc_conn_free (UCConn * conn);
 static UCTcpStatus connect_nonblock (const gint soc, struct sockaddr_in addr);
@@ -181,19 +184,22 @@ uc_conn_free (UCConn * conn)
  * @host: The host.
  * @port: The port.
  * @buffer: The HTTP request.
+ * @header_request: %TRUE if we need only to get the header
  *
  * Create a new #UCConn object.
  * 
  * Returns: A #UCConn node.
  */
 static UCConn *
-uc_conn_new (const gchar * proto, gchar * host, guint port, gchar * buffer)
+uc_conn_new (const gchar * proto, gchar * host, guint port, gchar * buffer,
+	     gboolean header_request)
 {
   UCConn *conn = NULL;
 
   conn = g_new0 (UCConn, 1);
   conn->thread_end = FALSE;
   conn->thread_ftp_end = FALSE;
+  conn->header_request = header_request;
   conn->timeout = FALSE;
   conn->socket = 0;
   conn->port = port;
@@ -297,6 +303,7 @@ uc_conn_disconnect (UCConn * conn)
  * @host: The host.
  * @port: The port.
  * @buffer: The HTTP request to send.
+ * @header_request: %TRUE if we need only to get the header
  *
  * Connect to a host:port, send a HTTP request and return the content
  * of the HTTP response.
@@ -305,7 +312,7 @@ uc_conn_disconnect (UCConn * conn)
  */
 GArray *
 uc_server_get_response (const gchar * proto, gchar * host, const guint port,
-			gchar * buffer)
+			gchar * buffer, gboolean header_request)
 {
   guint port_real = 0;
   gchar *host_real = NULL;
@@ -331,7 +338,7 @@ uc_server_get_response (const gchar * proto, gchar * host, const guint port,
       port_real = port;
     }
 
-  conn = uc_conn_new (proto, host_real, port_real, buffer);
+  conn = uc_conn_new (proto, host_real, port_real, buffer, header_request);
 
   /* connect */
   conn->thread_end = FALSE;
@@ -386,6 +393,7 @@ read_thread (void *data)
 {
   UCConn *conn = (UCConn *) data;
   gint nb_read = 0;
+  gchar *tmp = NULL;
   gchar buffer[UC_BUFFER_LEN + 1];
 
 #ifdef ENABLE_GNUTLS
@@ -565,8 +573,26 @@ read_thread (void *data)
   else if (conn->socket > 0)
     {
       write (conn->socket, conn->request, strlen (conn->request));
-      while ((nb_read = read (conn->socket, buffer, UC_BUFFER_LEN)) > 0)
-	g_array_append_vals (conn->array, buffer, nb_read);
+      while (!tmp &&
+	     (nb_read = read (conn->socket, buffer, UC_BUFFER_LEN)) > 0)
+	{
+	  if (conn->header_request)
+	    {
+	      gchar b = buffer[nb_read];
+	      buffer[nb_read] = '\0';
+	      if ((tmp = strstr (buffer, "\r\n\r\n")))
+		{
+		  *tmp = '\0';
+		  g_array_append_vals (conn->array, buffer, strlen (buffer));
+		  nb_read = 0;
+		}
+	      else
+		buffer[nb_read] = b;
+	    }
+
+	  if (nb_read > 0)
+	    g_array_append_vals (conn->array, buffer, nb_read);
+	}
 
       conn->tcp_status =
 	(nb_read == 0) ? UC_TCP_STATUS_OK : UC_TCP_STATUS_ERROR;
