@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2005 
+ * Copyright (C) 2002-2007
  * Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,7 @@
 
 void uc_conn_init (void);
 GArray *uc_server_get_response (const gchar * proto, gchar * host,
-				const guint port, gchar * buffer);
+				const guint port, gchar * buffer,
+				gboolean header_request);
 
 #endif
