@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2005 
+ * Copyright (C) 2002-2009
  * Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+#include <libgen.h>
 #include <gnet.h>
 #include "getopt.h"
 #include <ctype.h>
@@ -252,6 +253,13 @@
  */
 #define UC_STATUS_CODE_REDIRECTED \
 	"302"
+/** 
+ * UC_STATUS_CODE_REDIRECTED_SEE_OTHER:
+ *
+ * Macro for HTTP status code 303.
+ */
+#define UC_STATUS_CODE_REDIRECTED_SEE_OTHER \
+	"303"
 /**
  * UC_STATUS_CODE_RESTRICTED:
  *
@@ -545,9 +553,8 @@
  * Macro to indicate for which extensions we must search for virii.
  */
 #define UC_DEFAULT_SECURITY_VIRII_EXTENSIONS \
-	"pdf,doc,xls,ppt,swf,sxw,sxc,sxi," \
-	"tar,gz,tgz,zip,bz2,rar," \
-	"pif,exe,com,sys"
+	"application,audio,image,text,video," \
+	"chemical,model,multipart,x-conference,x-epoc,x-world"
 
 /**
  * UC_DEFAULT_SECURITY_VIRII_EXTENSIONS_LEN:
@@ -556,7 +563,7 @@
  * manage for virii.
  */
 #define UC_DEFAULT_SECURITY_VIRII_EXTENSIONS_LEN \
-	18
+	11
 
 /**
  * UC_DEFAULT_W3C_HTML_EXTENSIONS:
@@ -727,7 +734,6 @@
  */
 #define UC_BUFFER_LEN \
 	1024
-
 /**
  * UC_BUFFER_DATE_LEN:
  *
@@ -991,6 +997,7 @@ struct UCSearchState
 {
   gchar *string;
   gboolean case_sensitive;
+  gboolean status_code;
   gboolean pages_content;
   gboolean links_name;
   gboolean meta_content;
@@ -998,6 +1005,20 @@ struct UCSearchState
 }
 search_state;
 
+/**
+ * UCURLsUserActions:
+ * @id: Index of the action.
+ * @label: Action label.
+ * 
+ * Structure to store bad URLs actions.
+ */
+typedef struct _UCURLsUserActions UCURLsUserActions;
+struct _UCURLsUserActions
+{
+  gchar *id;
+  gchar *label;
+};
+
 /**
  * UCStatusCode:
  * @color: Color to associate with the status code.
@@ -1179,6 +1200,7 @@ struct _UCLinkProperties
   UCLinkType link_type;
   GdkPixbuf *link_icon;
   GdkPixbuf *status_icon;
+  gchar *user_action;
   gchar *link_value;
   gchar *url;
   gchar *normalized_url;
@@ -1204,43 +1226,8 @@ struct _UCLinkProperties
   GList *similar_links_parents;
   GList *bad_extensions;
   gboolean virii;
+  gchar *virname;
   gboolean w3c_valid;
   GList *tags;
 };
-
-/*********** begin of glibc code ***********/
-/**
- * GHashNode:
- * @key: Key.
- * @value: Value.
- * @next: Next #GHashNode.
- *  
- * Taken from glibc ghash.c to avoid use of
- * g_hash_table_foreach function.
- */
-typedef struct _GHashNode GHashNode;
-struct _GHashNode
-{
-  gpointer key;
-  gpointer value;
-  GHashNode *next;
-};
-
-/**
- * _GHashTable:
- * 
- * Taken from glibc ghash.c to avoid use of
- * g_hash_table_foreach function.
- */
-struct _GHashTable
-{
-  gint size;
-  gint nnodes;
-  GHashNode **nodes;
-  GHashFunc hash_func;
-  GEqualFunc key_equal_func;
-  GDestroyNotify key_destroy_func;
-  GDestroyNotify value_destroy_func;
-};
-/*********** end of glibc code ***********/
 #endif
